//---------------------------------------------------------------------------
// File: TMainForm.h
//
//  TDx_Sound_Library
//  Example1 - Enumerating Sound Devices
//
// Date: 9th January, 2004
// Author: Darren John Dwyer
// Copyright: (c) 2002-2004 Darren John Dwyer, All Rights Reserved.
//
// Description:
//
// This example shows how to enumerate available DirectSound devices,
// then how to create a selected device, retrieve it's capabilites,
// test it's playback, and finally, how to destroy the device and cleanup.
//
//---------------------------------------------------------------------------
#ifndef TMainFormH
#define TMainFormH
//---------------------------------------------------------------------------
#include <Classes.hpp>
#include <Controls.hpp>
#include <StdCtrls.hpp>
#include <Forms.hpp>
#include <Buttons.hpp>
#include <Graphics.hpp>
#include <vcl\ExtCtrls.hpp>
//---------------------------------------------------------------------------
#include "TDx_Sound.h"
#include "TDx_SoundBuffer.h"
#include "TDSBufferDesc.h"
#include "TDSCaps.h"
//---------------------------------------------------------------------------
#define MAX_DEVICES 256
//---------------------------------------------------------------------------
class TMainForm : public TForm
{
__published:    // IDE-managed Components

    // used for buffering toggle images
    TImage *Lo;
    TImage *Hi;
    TDx_Sound *Dx_Sound1;
    TDSCaps *DSCaps1;
    TDx_SoundBuffer *Dx_SoundBuffer1;
    TDSBufferDesc *DSBufferDesc1;

    // information
    TMemo *InfoMemo;
    TMemo *StatusMemo;

    TLabel *ActiveDeviceLabel;
    TLabel *TotalDevicesLabel;

    TListBox *CapsListBox;
    TListBox *DevicesListBox;

    // manual buttons
    TLabel *Enum;
    TLabel *Next;
    TLabel *Test;
    TLabel *Exit;
    TLabel *Clear;

    TImage *EnumImage;
    TImage *NextImage;
    TImage *TestImage;
    TImage *ExitImage;
    TImage *ClearImage;

    // Logos
    TImage *BCBTools;
    TImage *BCBTools_Anim;
    TBevel *Bevel2;
    TBevel *Bevel3;
    TBevel *Bevel4;

    TLabel *Label1;
    TLabel *Label2;
    TLabel *Label3;
    TLabel *Label4;
    TLabel *Label5;
    TLabel *Label6;
    TLabel *Label8;
    TLabel *Label9;
    TLabel *Label11;
    TLabel *Label12;
    TLabel *Label13;

    // TMainForm events
    void __fastcall FormCreate(TObject *Sender);
    void __fastcall FormDestroy(TObject *Sender);
    void __fastcall FormPaint(TObject *Sender);

    // Button events
    void __fastcall EnumClick(TObject *Sender);
    void __fastcall ClearClick(TObject *Sender);
    void __fastcall ExitClick(TObject *Sender);
    void __fastcall NextClick(TObject *Sender);
    void __fastcall TestClick(TObject *Sender);

    // Other events
    void __fastcall DevicesListBoxClick(TObject *Sender);
    void __fastcall ButtonsMouseMove(TObject *Sender,
          TShiftState Shift, int X, int Y);

    // TDx_Sound_Library events
    void __fastcall Dx_SoundBuffer1BufferLost(TObject *Sender);
    void __fastcall Dx_Sound1Create(TObject *Sender);
    void __fastcall Dx_Sound1Destroy(TObject *Sender);
    void __fastcall Dx_SoundBuffer1Create(TObject *Sender);
    void __fastcall Dx_Sound1DSEnumerate(TObject *Sender, GUID *Guid,
    AnsiString DriverDescription, AnsiString DriverModule, void *Context,
    bool &Finished);
    void __fastcall Dx_Sound1Error(TObject *Sender, AnsiString Function,
          AnsiString Error, AnsiString ErrorMessage);


private:    // User declarations

    // storage for TDx_SoundBuffer::OnDSEnumerate() data

    TList*            Guids;
    TStringList*    Devices;
    TStringList*    Modules;

    // button stuff
    AnsiString      Highlighted;

public:        // User declarations

    __fastcall TMainForm(TComponent* Owner);
};
//---------------------------------------------------------------------------
extern TMainForm *MainForm;
//---------------------------------------------------------------------------
#endif
