//---------------------------------------------------------------------------
// File: TMainForm.cpp
//
//  TDx_Sound_Library
//  Example4 - Playing Unlimited Sounds
//
// Date: 9th January, 2004
// Author: Darren John Dwyer
// Copyright: (c) 2002-2004 Darren John Dwyer, All Rights Reserved.
//
// Description:
//
// This example shows how to play any number of sound effects simultaneously.
//
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "TMainForm.h"
//---------------------------------------------------------------------------
#pragma link "TDx_Sound"
#pragma link "TDx_SoundBuffer"
#pragma link "TDSCaps"
#pragma link "TDSBufferDesc"
#pragma link "TDx_Sound_Library_Install"
#pragma resource "*.dfm"
//---------------------------------------------------------------------------
TMainForm *MainForm;
//---------------------------------------------------------------------------
__fastcall TMainForm::TMainForm(TComponent* Owner)
    : TForm(Owner)
{
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormCreate(TObject *Sender)
{
KnownDevices = new TStringList();
KnownModules = new TStringList();
KnownGuids = new TList();

DevicesListBox->Items->Clear();
if (Dx_Sound1->DSEnumerate( NULL ))
    if (KnownDevices->Count>0)
        {
        DevicesListBox->ItemIndex = 0;
        Dx_Sound1->Create( (GUID*) KnownGuids->Items[DevicesListBox->ItemIndex] );
        }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormDestroy(TObject *Sender)
{
Timer1->Enabled = false;
if (Dx_Sound1->Created) Dx_Sound1->Destroy();

if (KnownGuids) delete KnownGuids;
if (KnownModules) delete KnownModules;
if (KnownDevices) delete KnownDevices;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::DevicesListBoxClick(TObject *Sender)
{
if (Dx_Sound1->Created) Dx_Sound1->Destroy();
NotCreatedLabel->Visible = !Dx_Sound1->Create( (GUID*) KnownGuids->Items[DevicesListBox->ItemIndex] );
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Sound1Destroy(TObject *Sender)
{
if (BufferNames)
    {
    for (int i=0;i<BufferNames->Count;i++) BufferNames->Delete(i);
    delete BufferNames; BufferNames = NULL;
    }
if (BufferDescs)
    {
    for (int i=0;i<BufferDescs->Count;i++)
        {
        TDSBufferDesc* temp_buffer_desc = (TDSBufferDesc*) BufferDescs->Items[i];
        delete temp_buffer_desc;
        }
    delete BufferDescs; BufferDescs = NULL;
    }
if (SoundBuffers)
    {
    for (int i=0;i<SoundBuffers->Count;i++)
        {
        TDx_SoundBuffer* temp_sound_buffer = (TDx_SoundBuffer*) SoundBuffers->Items[i];
        delete temp_sound_buffer;
        }
    delete SoundBuffers; SoundBuffers = NULL;
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::PlayButtonClick(TObject *Sender)
{
if (!SoundBuffers || !BufferDescs || !BufferNames) return;

// scan through all current selections
for (int i=0;i<FileListBox1->Items->Count;i++)
    if (FileListBox1->Selected[i])
        {
        // scan through to make sure not re-creating an existing buffer
        int exists_at = -1;
        for (int j=0;j<BufferNames->Count;j++)
            if (FileListBox1->Items->Strings[i]==BufferNames->Strings[j])
                {
                exists_at = j;
                break;
                }

        // if it's already created
        if (exists_at!=-1)
            {
            TDx_SoundBuffer* temp_sound_buffer = (TDx_SoundBuffer*) SoundBuffers->Items[exists_at];
            // it's already created; if it's not playing, play it now
            if (!temp_sound_buffer->Playing)
                if (LoopingCheckBox->Checked) temp_sound_buffer->Play(0,DSBPLAY_LOOPING);
                else temp_sound_buffer->Play(0,0);
            }
        else
            // it's NOT already created
            {
            // create new objects
            TDx_SoundBuffer* temp_sound_buffer = new TDx_SoundBuffer(this);
            if (!temp_sound_buffer) return;

            TDSBufferDesc* temp_buffer_desc = new TDSBufferDesc(this);
            if (!temp_buffer_desc)
                {
                delete temp_sound_buffer;
                return;
                }
            temp_buffer_desc->Flags = DSBCAPS_CTRLFREQUENCY | DSBCAPS_CTRLPAN | DSBCAPS_CTRLVOLUME |
                                    DSBCAPS_GETCURRENTPOSITION2 | DSBCAPS_GLOBALFOCUS;

            // add to list
            SoundBuffers->Add( temp_sound_buffer );
            BufferDescs->Add( temp_buffer_desc );
            BufferNames->Add( FileListBox1->Items->Strings[i] );

            // create .wav buffer
            temp_sound_buffer->CreateFromFile( FileListBox1->Items->Strings[i], temp_buffer_desc, Dx_Sound1 );

            // play sound
            if (LoopingCheckBox->Checked) temp_sound_buffer->Play(0,DSBPLAY_LOOPING);
            else temp_sound_buffer->Play(0,0);
            }
        }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::StopButtonClick(TObject *Sender)
{
if (!SoundBuffers || !BufferDescs || !BufferNames) return;

// scan through all current selections
for (int i=0;i<FileListBox1->Items->Count;i++)
    if (FileListBox1->Selected[i])
        {
        // scan through to make sure buffer exists
        int exists_at = -1;
        for (int j=0;j<BufferNames->Count;j++)
            if (FileListBox1->Items->Strings[i]==BufferNames->Strings[j])
                {
                exists_at = j;
                break;
                }

        // if .wav buffer exists, delete it
        if (exists_at!=-1)
            {
            TDSBufferDesc* temp_buffer_desc = (TDSBufferDesc*) BufferDescs->Items[exists_at];
            TDx_SoundBuffer* temp_sound_buffer = (TDx_SoundBuffer*) SoundBuffers->Items[exists_at];

            delete temp_buffer_desc;
            delete temp_sound_buffer;

            BufferNames->Delete(exists_at);
            BufferDescs->Delete(exists_at); BufferDescs->Pack();
            SoundBuffers->Delete(exists_at); SoundBuffers->Pack();
            }
        }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FileListBox1DblClick(TObject *Sender)
{
PlayButtonClick(this);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Timer1Timer(TObject *Sender)
{
if (!SoundBuffers || !BufferDescs || !BufferNames) return;

// destroy any existing soundbuffers that have stopped playing
for (int i=SoundBuffers->Count-1;i>=0;i--)
    {
    TDSBufferDesc* temp_buffer_desc = (TDSBufferDesc*) BufferDescs->Items[i];
    TDx_SoundBuffer* temp_sound_buffer = (TDx_SoundBuffer*) SoundBuffers->Items[i];

    if (!temp_sound_buffer->Playing)
        {
        temp_sound_buffer->Destroy();
        delete temp_buffer_desc;
        delete temp_sound_buffer;

        BufferNames->Delete(i);
        BufferDescs->Delete(i); BufferDescs->Pack();
        SoundBuffers->Delete(i); SoundBuffers->Pack();
        }
    }

// scan through display lists & update buffer status
TStringList* temp_name = new TStringList();
TStringList* temp_size = new TStringList();
TStringList* temp_status = new TStringList();
TStringList* temp_position = new TStringList();

if (temp_name && temp_size && temp_status && temp_position)
    {
    // scan through current display rows, checking each row to see if it
    // corresponds to a currently created soundbuffer.
    // If so, add updated information to new lists for currently created buffers
    for (int i=0;i<NameMemo->Lines->Count;i++)
        for (int j=0;j<SoundBuffers->Count;j++)
            if (NameMemo->Lines->Strings[i]==BufferNames->Strings[j])
                {
                TDx_SoundBuffer* temp_sound_buffer = (TDx_SoundBuffer*) SoundBuffers->Items[j];
                TDSBufferDesc* temp_buffer_desc = (TDSBufferDesc*) BufferDescs->Items[j];

                if (temp_sound_buffer->Created)
                    {
                    dword temp_play_pos, temp_write_pos;
                    temp_sound_buffer->GetCurrentPosition( &temp_play_pos, &temp_write_pos);

                    temp_name->Add( BufferNames->Strings[j].SubString(1,14) );
                    temp_size->Add( IntToStr(temp_buffer_desc->BufferBytes) );
                    temp_position->Add( IntToStr(temp_play_pos) );

                    if (temp_sound_buffer->Playing) temp_status->Add( "Playing..." );
                    else temp_status->Add( "Stopped..." );
                    }
                break;
                }

    // scan through created soundbuffers, checking each display row to see if it
    // exists. If not, add information to new lists
    for (int j=0;j<SoundBuffers->Count;j++)
           {
        TDx_SoundBuffer* temp_sound_buffer = (TDx_SoundBuffer*) SoundBuffers->Items[j];
        TDSBufferDesc* temp_buffer_desc = (TDSBufferDesc*) BufferDescs->Items[j];

        bool exists = false;
        for (int i=0;i<NameMemo->Lines->Count;i++)
            if (NameMemo->Lines->Strings[i]==BufferNames->Strings[j])
                {
                exists = true;
                break;
                }
        if (!exists && temp_sound_buffer->Created)
            {
            if (temp_sound_buffer->Created)
                {
                dword temp_play_pos, temp_write_pos;
                temp_sound_buffer->GetCurrentPosition( &temp_play_pos, &temp_write_pos);

                temp_name->Add( BufferNames->Strings[j].SubString(1,14) );
                temp_size->Add( IntToStr(temp_buffer_desc->BufferBytes) );
                temp_position->Add( IntToStr(temp_play_pos) );

                if (temp_sound_buffer->Playing) temp_status->Add( "Playing..." );
                else temp_status->Add( "Stopped..." );
                }
            }
        }

    // update the display memos
    NameMemo->Lines->Clear();
    NameMemo->Lines->AddStrings( temp_name );
    SizeMemo->Lines->Clear();
    SizeMemo->Lines->AddStrings( temp_size );
    StatusMemo->Lines->Clear();
    StatusMemo->Lines->AddStrings( temp_status );
    PositionMemo->Lines->Clear();
    PositionMemo->Lines->AddStrings( temp_position );
    }

SoundsCurrentlyPlaying->Caption = "Sounds Currently Playing: "+IntToStr(NameMemo->Lines->Count);

if (temp_position) delete temp_position;
if (temp_status) delete temp_status;
if (temp_size) delete temp_size;
if (temp_name) delete temp_name;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Sound1Create(TObject *Sender)
{
Dx_Sound1->SetCooperativeLevel(Application->Handle, DSSCL_PRIORITY);

SoundBuffers = new TList();
BufferDescs = new TList();
BufferNames = new TStringList();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ExitButtonClick(TObject *Sender)
{
Close();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SelectAllButtonClick(TObject *Sender)
{
for (int i=0;i<FileListBox1->Items->Count;i++) FileListBox1->Selected[i] = true;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Sound1DSEnumerate(TObject *Sender, GUID *Guid,
    AnsiString DriverDescription, AnsiString DriverModule, void *Context,
    bool &Finished)
{
if (KnownDevices && KnownModules && KnownGuids)
    {
    KnownDevices->Add( DriverDescription );
    KnownModules->Add( DriverModule );
    KnownGuids->Add( (void*) Guid );
    DevicesListBox->Items->Add( DriverDescription + ", ("+DriverModule+")" );
    }
else
    Finished = true;
}
//---------------------------------------------------------------------------
